library fields.transform;

/// Check if a transform from the [sourceType] to the [targetType] can be 
/// performed.  Returns _true_ if such a transform is possible, else _false_.
bool canTransform(Type sourceType, Type targetType) {
  if (sourceType == targetType) return true;
  return Transform._transforms.any(
      (t) => t.sourceType == sourceType && t.targetType == targetType);
}

/// Transform the given [value] from its [sourceType] to a [targetType].
/// Returns the transformed value.
transform(value, Type sourceType, Type targetType) {
  if (sourceType == targetType) return value;
  return Transform._transforms.firstWhere(
      (t) => t.sourceType == sourceType && t.targetType == targetType)(value);
}

/// An error encountered during a transform.
class TransformError extends Error {
  /// The caught object that caused the transform to err; may be `null` if none.
  final cause;
  /// A description of this transform error.
  final String message;
  /// Constructs a new transform error with the optional [cause] and [message].
  TransformError({this.cause: null, this.message: ''});
  String toString() => 'TransformError: $message |> $cause';
}

/// A one-directional transform from type [S] to type [T].
abstract class Transform<S, T> {
  static final Set<Transform> _transforms = new Set<Transform>();
  
  static void register(Transform transform) => _transforms.add(transform);
  
  /// Unregister the given [transform].
  /// Returns _true_ if the [transform] was registered, else returns _false_.
  static bool unregister(Transform transform) => _transforms.remove(transform);
  
  static void unregisterAll() => _transforms.clear();
  
  /// The source type of this transform.
  final Type sourceType = S;
  /// The target type of this transform.
  final Type targetType = T;
  const Transform();
  /// Call this transform on [value].  Returns the transformed value.
  T call(S value);
}
